@@ -31,6 +31,8 @@
 
 using namespace std;
 
+#include <games-stock.h>
+
 #include "blackjack.h"
 #include "menu.h"
 #include "dialog.h"
@@ -144,35 +146,25 @@ on_help_about_activate (GtkAction *action,
         };
 
         const gchar *documenters[] = {
+                "William Jon McCann <mccann@jhu.edu>",
                 NULL
         };
-
-        const gchar *translator_credits = _("translator-credits");
-
-        {
-                int i=0;
-                while (authors[i] != NULL) { authors[i]=_(authors[i]); i++; }
-        }
-
-	char *filename = NULL;
-	filename = g_build_filename (PIXMAPDIR, "gnome-blackjack.png", NULL);
-
-        GdkPixbuf *pixbuf = NULL;
-        if (filename != NULL) {
-                pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
-                g_free (filename);
-        }
+	gchar *license = games_get_license (_("Blackjack"));
 
         gtk_show_about_dialog (GTK_WINDOW (toplevel_window),
-                               "logo", pixbuf,
                                "authors", authors,
                                "documenters", documenters,
                                "version", VERSION,
-                               "copyright", "Copyright \xc2\xa9 2003-2004 William Jon McCann, Eric Farmer",
-                               "comments", _("Blackjack provides a casino-style blackjack card game."),
+                               "copyright", "Copyright \xc2\xa9 2003-2006 William Jon McCann, Eric Farmer",
+                               "comments", _("Blackjack is a casino-style card game."),
+                               "logo-icon-name", "gnome-blackjack",
+                               "license", license,
+                               "translator_credits", _("translator-credits"),
+                               "logo-icon-name", "gnome-aisleriot.png",
+                               "website", "http://www.gnome.org/projects/gnome-games/",
+                               "wrap-license", TRUE,
                                NULL);
-
-        g_object_unref (pixbuf);
+	g_free (license);
 
         return;
 }
