@@ -1,3 +1,9 @@
+try:
+    import pygtk
+    pygtk.require('2.0')
+except ImportError, err:
+    print ("PyGTK not found. Please make sure it is installed properly and referenced in your PYTHONPATH environment variable.")
+
 import gtk, gobject, gtk.glade
 import gnome, gnome.ui, pango
 import os, os.path
@@ -112,7 +118,6 @@ class UI (gconf_wrapper.GConfWrapper):
                      'always_show_hints':0,
                      'player':os.environ.get('USERNAME',''),
                      'difficulty':0.0,
-                     'minimum_font_size':pango.SCALE * 7, # minimum font-size
                      'minimum_number_of_new_puzzles':MIN_NEW_PUZZLES,
                      'bg_black':1,
                      'bg_custom_color':'',
@@ -137,6 +142,7 @@ class UI (gconf_wrapper.GConfWrapper):
         gnome.program_init('gnome-sudoku',VERSION,
                            properties={gnome.PARAM_APP_DATADIR:APP_DATA_DIR}
                            )
+	self.w.set_title(APPNAME);
         self.w.connect('delete-event',self.quit_cb)
         self.vb = gtk.VBox()
         self.uimanager = gtk.UIManager()
